"use strict";

var t = require('@babel/types');

module.exports = function convertAstExpressionToVariable(node) {
  if (t.isObjectExpression(node)) {
    var obj = {};
    var properties = node.properties;
    properties.forEach(function (property) {
      if (property.type === 'ObjectProperty' || property.type === 'ObjectMethod') {
        var key = convertAstExpressionToVariable(property.key);
        var value = convertAstExpressionToVariable(property.value);
        obj[key] = value;
      }
    });
    return obj;
  } else if (t.isArrayExpression(node)) {
    return node.elements.map(convertAstExpressionToVariable);
  } else if (t.isLiteral(node)) {
    return node.value;
  } else if (t.isIdentifier(node) || t.isJSXIdentifier(node)) {
    var name = node.name;
    return name === 'undefined' ? undefined : name;
  } else if (t.isJSXExpressionContainer(node)) {
    return convertAstExpressionToVariable(node.expression);
  }
};