"use strict";

var t = require('@babel/types');

var _require = require('loader-utils'),
    getOptions = _require.getOptions;

var parse = require('./parse');

var traverse = require('./traverse');

var getDefaultExportedPath = require('./getDefaultExportedPath');

var convertAstExpressionToVariable = require('./convertAstExpressionToVariable');

var historyMemory = {
  hash: 'createHashHistory',
  memory: 'createMemoryHistory',
  browser: 'createBrowserHistory'
};
/**
 * Universal App Shell Loader for Rax.
 */

module.exports = function (content) {
  var options = getOptions(this) || {};

  if (this.data.appConfig !== null) {
    var _this$data$appConfig = this.data.appConfig,
        _this$data$appConfig$ = _this$data$appConfig.pages,
        pages = _this$data$appConfig$ === void 0 ? [] : _this$data$appConfig$,
        _this$data$appConfig$2 = _this$data$appConfig.historyType,
        historyType = _this$data$appConfig$2 === void 0 ? 'hash' : _this$data$appConfig$2;
    var fixRootStyle = '';
    /**
     * Weex only support memory history.
     */

    if (options.type === 'weex') historyType = 'memory';
    /**
     * Web only compatible with 750rpx.
     */

    if (options.type === 'web') {
      var mutiple = options.mutiple || 100;
      fixRootStyle = "\n        const html = document.documentElement;\n        html.style.fontSize = html.clientWidth / 750 * " + mutiple + " + 'px';\n      ";
    }

    var requestPages = pages.map(function (path, index) {
      return "import Page" + index + " from './" + path + "';";
    }).join('\n');
    var assembleRoutes = pages.map(function (path, index) {
      var extra = '';

      if (index === 0) {
        extra = "routes.push({ \n          path: '/', \n          component: () => createElement(Redirect, { to: '/" + path + "' }),\n        });";
      }

      return extra + ("routes.push({\n        path: '/" + path + "',\n        component: () => createElement(Page" + index + ", { app: appInstance }),\n      });");
    }).join('\n');
    var source = "\n      import App from '" + this.resourcePath + "';\n      import { render, createElement, useLayoutEffect } from 'rax';\n      import { useRouter, replace } from 'rax-use-router';\n      import { " + historyMemory[historyType] + " as createHistory } from 'history';\n      import DriverUniversal from 'driver-universal';\n      " + requestPages + "\n      \n      const appInstance = new App();\n      const getRouteConfig = () => {\n        const routes = [];\n        " + assembleRoutes + "\n        return {\n          history: createHistory(),\n          routes,\n        };\n      };\n      \n      function Redirect(props) {\n        useLayoutEffect(() => {\n          replace(props.to);\n        }, []);\n      }\n      \n      function Entry() {\n        const { component } = useRouter(getRouteConfig);\n        return component;\n      }\n      \n      " + fixRootStyle + "\n      render(createElement(Entry), null, { driver: DriverUniversal });\n    ";
    return source;
  } else {
    return content;
  }
};

module.exports.pitch = function (remainingRequest, precedingRequest, data) {
  data.appConfig = null; // Provide default value.

  var rawContent = this.fs.readFileSync(this.resource).toString();
  var ast = parse(rawContent);
  var defaultExportedPath = getDefaultExportedPath(ast);

  if (defaultExportedPath) {
    traverse(defaultExportedPath, {
      ClassProperty: function ClassProperty(path) {
        var node = path.node;

        if (node.static && t.isIdentifier(node.key, {
          name: 'config'
        })) {
          data.appConfig = convertAstExpressionToVariable(node.value);
        }
      }
    });
  }
};